#
# xxx - Example of a Scipy module, see DEVELOPERS.txt 
#

# Get documentation string:
from info_xxx import __doc__

# Import symbols from sub-module:
from foo import *

# Import sub-package:
import yyy

# Import extension module
import spam
