"""
foo - Scipy module xxx submodule.
"""

__all__ = ['xxx_foo_bar','foo_gun']

def xxx_foo_bar():
    """
    Scipy module xxx global (see info_xxx.py) test function.
    """
    return "Hello from xxx_foo_bar"

def foo_gun():
    """
    Scipy module xxx test function.
    """
    return "Hello from foo_gun"

