#!/usr/bin/env python
#
# Created by: Pearu Peterson, October 2003
#

import sys
from scipy_test.testing import *
set_package_path()
import linalg.atlas_version
del sys.path[0]

# No tests possible.
# Importing atlas_version will print out atlas version.
