/*
    Multipack project.
    This file is generated by setmodules.py. Do not modify it.
 */
#include "multipack.h"
static PyObject *fitpack_error;
#include "__fitpack.h"
static struct PyMethodDef fitpack_module_methods[] = {
{"_curfit", fitpack_curfit, METH_VARARGS, doc_curfit},
{"_spl_", fitpack_spl_, METH_VARARGS, doc_spl_},
{"_splint", fitpack_splint, METH_VARARGS, doc_splint},
{"_sproot", fitpack_sproot, METH_VARARGS, doc_sproot},
{"_spalde", fitpack_spalde, METH_VARARGS, doc_spalde},
{"_parcur", fitpack_parcur, METH_VARARGS, doc_parcur},
{"_surfit", fitpack_surfit, METH_VARARGS, doc_surfit},
{"_bispev", fitpack_bispev, METH_VARARGS, doc_bispev},
{NULL,		NULL, 0, NULL}
};
void init_fitpack(void) {
  PyObject *m, *d, *s;
  m = Py_InitModule("_fitpack", fitpack_module_methods);
  import_array();
  d = PyModule_GetDict(m);

  s = PyString_FromString(" 1.6 ");
  PyDict_SetItemString(d, "__version__", s);
  fitpack_error = PyErr_NewException ("fitpack.error", NULL, NULL);
  Py_DECREF(s);
  if (PyErr_Occurred())
    Py_FatalError("can't initialize module fitpack");
}
        
