# Copyright (c) 1996, 1997, The Regents of the University of California.
# All rights reserved.  See Legal.htm for full text and disclaimer.

Graph2dType = "<type 'graph2d'>"
Graph3dType = "<type 'graph3d'>"
SurfaceType = "<type 'surface'>"
Mesh3dType = "<type 'mesh3d'>"
LinesType = "<type 'lines'>"
PolyMapType = "<type 'polymap'>"
QuadMeshType = "<type 'quadmesh'>"
RegionType = "<type 'region'>"
GistType = "<type 'Gist'>"
NarType = "<type 'Nar'>"
Animation2dType = "<type 'animation2d'>"
CellArrayType = "<type 'cellarray'>"
CurveType = "<type 'curvetype'>"
Slice3dType = "<type 'slicetype'>"
PlaneType = "<type 'plane'>"
