// From: Doug Moore <dougm@rice.edu>
// Organization: Rice University Computer Science
// SC4.0 18 Oct 1995 C++ 4.1

/******************************************************************************
 * config.h           Compiler language support flags
 *
 * This file was generated automatically by the script bzconfig.
 * You should rerun bzconfig each time you switch compilers, install new
 * standard libraries, or change compiler versions.
 *
 */
 
 
#ifndef BZ_CONFIG_H
#define BZ_CONFIG_H
 
#define BZ_COMPILER_NAME   "CC"
#define BZ_OS_NAME         "SunOS 5.5"
#define BZ_BZCONFIG_DATE   "Fri Nov  8 16:56:17 CST 1996"
 
#undef  BZ_NAMESPACES
#undef  BZ_EXCEPTIONS
#undef  BZ_RTTI
#undef  BZ_MEMBER_CONSTANTS
#define BZ_OLD_FOR_SCOPING
#undef  BZ_EXPLICIT
#undef  BZ_MUTABLE
#undef  BZ_TYPENAME
#undef  BZ_NCEG_RESTRICT
#undef  BZ_BOOL
#undef  BZ_CONST_CAST
#undef  BZ_STATIC_CAST
#undef  BZ_REINTERPRET_CAST
#undef  BZ_DYNAMIC_CAST
#define BZ_TEMPLATES
#undef  BZ_PARTIAL_SPECIALIZATION
#undef  BZ_DEFAULT_TEMPLATE_PARAMETERS
#undef  BZ_MEMBER_TEMPLATES
#undef  BZ_MEMBER_TEMPLATES_OUTSIDE_CLASS
#undef  BZ_FULL_SPECIALIZATION_SYNTAX
#undef  BZ_FUNCTION_NONTYPE_PARAMETERS
#undef  BZ_TEMPLATE_QUALIFIED_BASE_CLASS
#undef  BZ_TEMPLATE_QUALIFIED_RETURN_TYPE
#undef  BZ_EXPLICIT_TEMPLATE_FUNCTION_QUALIFICATION
#undef  BZ_TEMPLATES_AS_TEMPLATE_ARGUMENTS
#undef  BZ_TEMPLATE_KEYWORD_QUALIFIER
#define BZ_TYPE_PROMOTION
#undef  BZ_HAVE_COMPLEX
#undef  BZ_HAVE_NUMERIC_LIMITS
#undef  BZ_HAVE_VALARRAY
 
#endif // BZ_CONFIG_H

