/*
 * resource.c -- $Id: resource.c,v 1.1 2003/03/08 15:26:51 travo Exp $
 * X11 runtime resources
 *
 * Copyright (c) 1998.  See accompanying LEGAL file for details.
 */

#include "config.h"
#include "playx.h"

char *x_xfont = 0;       /* boldfont, font, Font */
char *x_foreground = 0;  /* foreground, Foreground */
char *x_background = 0;  /* background, Background */
char *x_guibg = 0;       /* guibg */
char *x_guifg = 0;       /* guifg */
char *x_guihi = 0;       /* guihi */
char *x_guilo = 0;       /* guilo */
