/******************************************************************************
 * config.h           Compiler language support flags
 *
 * This file was generated automatically by the script bzconfig.
 * You should rerun bzconfig each time you switch compilers, install new
 * standard libraries, or change compiler versions.
 *
 */

 
#ifndef BZ_CONFIG_H
#define BZ_CONFIG_H
 
#define BZ_COMPILER_NAME   "g++"
#define BZ_OS_NAME         "AIX 1"
#define BZ_BZCONFIG_DATE   "Wed Nov  6 14:41:20 EST 1996"
 
#undef  BZ_NAMESPACES
#define BZ_EXCEPTIONS
#define BZ_RTTI
#define BZ_MEMBER_CONSTANTS
#define BZ_OLD_FOR_SCOPING
#define BZ_EXPLICIT
#define BZ_MUTABLE
#define BZ_TYPENAME
#undef  BZ_NCEG_RESTRICT
#define BZ_BOOL
#define BZ_CONST_CAST
#define BZ_STATIC_CAST
#define BZ_REINTERPRET_CAST
#define BZ_DYNAMIC_CAST
#define BZ_TEMPLATES
#undef  BZ_PARTIAL_SPECIALIZATION
#undef  BZ_DEFAULT_TEMPLATE_PARAMETERS
#undef  BZ_MEMBER_TEMPLATES
#undef  BZ_MEMBER_TEMPLATES_OUTSIDE_CLASS
#undef  BZ_FULL_SPECIALIZATION_SYNTAX
#undef  BZ_FUNCTION_NONTYPE_PARAMETERS
#define BZ_TEMPLATE_QUALIFIED_BASE_CLASS
#undef  BZ_TEMPLATE_QUALIFIED_RETURN_TYPE
#undef  BZ_EXPLICIT_TEMPLATE_FUNCTION_QUALIFICATION
#undef  BZ_TEMPLATES_AS_TEMPLATE_ARGUMENTS
#undef  BZ_TEMPLATE_KEYWORD_QUALIFIER
#define BZ_TYPE_PROMOTION
#define BZ_HAVE_COMPLEX
#undef  BZ_HAVE_NUMERIC_LIMITS
#undef  BZ_HAVE_VALARRAY
 
#endif // BZ_CONFIG_H
