"""
Vector Quantization / Kmeans
============================

    Clustering Algorithms are useful in Information Theory, target detection,
    communications, compression, and other areas.  Currently only Vector
    Quantization and the KMeans algorithm are supported by the vq module.
    Self Organized Feature Maps (SOM) and other approaches are also scheduled
    to appear at a theater near you.
"""
