"""
Scipy example module for developers

This is documentation of xxx.
Provides:
  xxx_foo_bar - also available in scipy name space
  foo_gun
  yyy.fun
"""

global_symbols = ['xxx_foo_bar']

postpone_import = 1
