#
# linalg - Linear algebra routines
#

from info_linalg import __doc__
from linalg_version import linalg_version as __version__

from basic import *
from decomp import *
from matfuncs import *
from blas import *
from iterative import *

